import { InteractiveSection } from './InteractiveSection';
import { Markdown } from './Markdown';
import { OpenAPICopyButton } from './OpenAPICopyButton';
import { OpenAPISchemaName } from './OpenAPISchemaName';
import { t } from './translate';
import { createStateKey, resolveDescription } from './utils';
/**
 * Present securities authorization that can be used for this operation.
 */
export function OpenAPISecurities(props) {
    var securities = props.securities, context = props.context;
    if (securities.length === 0) {
        return null;
    }
    return (<InteractiveSection header={t(context.translation, 'authorizations')} stateKey={createStateKey('securities', context.blockKey)} toggeable defaultOpened={false} toggleIcon={context.icons.chevronRight} selectIcon={context.icons.chevronDown} className="openapi-securities" tabs={securities.map(function (_a) {
            var key = _a[0], security = _a[1];
            var description = resolveDescription(security);
            return {
                key: key,
                label: key,
                body: (<div className="openapi-schema">
                            <div className="openapi-schema-presentation">
                                {getLabelForType(security, context)}

                                {description ? (<Markdown source={description} className="openapi-securities-description"/>) : null}
                            </div>
                        </div>),
            };
        })}/>);
}
function getLabelForType(security, context) {
    var _a;
    switch (security.type) {
        case 'apiKey':
            return (<OpenAPISchemaName context={context} propertyName={(_a = security.name) !== null && _a !== void 0 ? _a : 'apiKey'} type="string" required={security.required}/>);
        case 'http':
            if (security.scheme === 'basic') {
                return (<OpenAPISchemaName context={context} propertyName="Authorization" type="string" required={security.required}/>);
            }
            if (security.scheme === 'bearer') {
                var description = resolveDescription(security);
                return (<>
                        <OpenAPISchemaName context={context} propertyName="Authorization" type="string" required={security.required}/>
                        {/** Show a default description if none is provided */}
                        {!description ? (<Markdown source={"Bearer authentication header of the form Bearer ".concat('&lt;token&gt;', ".")} className="openapi-securities-description"/>) : null}
                    </>);
            }
            return (<OpenAPISchemaName context={context} propertyName="HTTP" required={security.required}/>);
        case 'oauth2':
            return <OpenAPISchemaOAuth2Flows context={context} security={security}/>;
        case 'openIdConnect':
            return (<OpenAPISchemaName context={context} propertyName="OpenID Connect" required={security.required}/>);
        default:
            // @ts-ignore
            return security.type;
    }
}
function OpenAPISchemaOAuth2Flows(props) {
    var _a;
    var context = props.context, security = props.security;
    var flows = Object.entries((_a = security.flows) !== null && _a !== void 0 ? _a : {});
    return (<div className="openapi-securities-oauth-flows">
            {flows.map(function (_a, index) {
            var name = _a[0], flow = _a[1];
            return (<OpenAPISchemaOAuth2Item key={index} flow={flow} name={name} context={context} security={security}/>);
        })}
        </div>);
}
function OpenAPISchemaOAuth2Item(props) {
    var _a;
    var flow = props.flow, context = props.context, security = props.security, name = props.name;
    if (!flow) {
        return null;
    }
    var scopes = Object.entries((_a = flow === null || flow === void 0 ? void 0 : flow.scopes) !== null && _a !== void 0 ? _a : {});
    return (<div>
            <OpenAPISchemaName context={context} propertyName="OAuth2" type={name} required={security.required}/>
            <div className="openapi-securities-oauth-content openapi-markdown">
                {security.description ? <Markdown source={security.description}/> : null}
                {'authorizationUrl' in flow && flow.authorizationUrl ? (<span>
                        Authorization URL:{' '}
                        <OpenAPICopyButton value={flow.authorizationUrl} context={context} className="openapi-securities-url" withTooltip>
                            {flow.authorizationUrl}
                        </OpenAPICopyButton>
                    </span>) : null}
                {'tokenUrl' in flow && flow.tokenUrl ? (<span>
                        Token URL:{' '}
                        <OpenAPICopyButton value={flow.tokenUrl} context={context} className="openapi-securities-url" withTooltip>
                            {flow.tokenUrl}
                        </OpenAPICopyButton>
                    </span>) : null}
                {'refreshUrl' in flow && flow.refreshUrl ? (<span>
                        Refresh URL:{' '}
                        <OpenAPICopyButton value={flow.refreshUrl} context={context} className="openapi-securities-url" withTooltip>
                            {flow.refreshUrl}
                        </OpenAPICopyButton>
                    </span>) : null}
                {scopes.length ? (<div>
                        {t(context.translation, 'available_scopes')}:{' '}
                        <ul>
                            {scopes.map(function (_a) {
                var key = _a[0], value = _a[1];
                return (<li key={key}>
                                    <OpenAPICopyButton value={key} context={context} withTooltip>
                                        <code>{key}</code>
                                    </OpenAPICopyButton>
                                    : {value}
                                </li>);
            })}
                        </ul>
                    </div>) : null}
            </div>
        </div>);
}
